from setuptools import setup

setup(name='pyspotless',
      version='1.0.0',
      description='A client library for integrating spotlessdata.com into Python applications',
      url='https://spotlessdata.com/docs/using-spotless-python',
      author='Spotless Data',
      author_email='team@spotlessdata.com',
      license='MIT',
      packages=['pyspotless'],
      zip_safe=False,
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Intended Audience :: Developers',
                   'Topic :: Software Development :: Libraries',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6'],
      keywords='spotless datacleansing',
      test_suite='nose.collector',
      ptests_require=['nose'],)
